import requests
from bs4 import BeautifulSoup

def get_ifc_bridge_part_type_enum(url):
    try:
        # Send an HTTP GET request to the URL
        response = requests.get(url)
        response.raise_for_status()  # Raise an error for bad status codes

        # Parse the HTML content using BeautifulSoup
        soup = BeautifulSoup(response.text, 'html.parser')

        # Find the <tbody> containing all <tr> elements
        tbody = soup.find('tbody')
        if not tbody:
            print("Could not find the <tbody> section in the HTML.")
            return []

        # Find all <tr> tags within the <tbody>
        table_rows = tbody.find_all('tr')

        # Initialize an empty list to store enumeration values
        enum_values = []

        # Iterate through each table row and extract the enumeration name
        for row in table_rows:
            # Find all <td> tags in the row
            cols = row.find_all('td')
            if len(cols) >= 1:
                # The first <td> contains the enumeration name within a <code> tag
                name_td = cols[0]
                code_tag = name_td.find('code')
                if code_tag:
                    enum_name = code_tag.get_text(strip=True)
                    enum_values.append(enum_name)

        # Check if any enumeration values were found
        if enum_values:
            return enum_values
        else:
            print("No enumeration values found in the <tbody> section.")
            return []

    except requests.exceptions.RequestException as e:
        print(f"An error occurred while fetching the URL: {e}")
        return []

    except requests.exceptions.RequestException as e:
        print(f"An error occurred while fetching the URL: {e}")
        return []

if __name__ == "__main__":
    enum_url = "https://standards.buildingsmart.org/IFC/RELEASE/IFC4_3/HTML/lexical/IfcBridgePartTypeEnum.htm"
    enum_name = "IfcBridgePartTypeEnum"
    bridge_part_type_enum_list = get_ifc_bridge_part_type_enum(enum_url)
    print("This is the list",bridge_part_type_enum_list)
