import ifcopenshell
import ifcopenshell.util
import ifcopenshell.guid

from ifc_bridge_creation import *
from buildingSmart_types import get_ifc_bridge_part_type_enum

URL = "https://standards.buildingsmart.org/IFC/RELEASE/IFC4_3/HTML/lexical/IfcBridgePartTypeEnum.htm"

def find_bridge_part(bridge, part_type):
    '''
    
    '''
    if not bridge.IsDecomposedBy:
        return None
    part_type_upper = part_type.upper() if part_type else "NOTDEFINED"
    
    for rel in bridge.IsDecomposedBy:
        for obj in rel.RelatedObjects:
            if obj.is_a("IfcBridgePart"):
                actual_predefined_type = getattr(obj, "PredefinedType",None)
                actual_object_type = getattr(obj, "ObjectType",None) or ""
                if actual_predefined_type == part_type_upper:
                    return obj
                if actual_predefined_type == "USERDEFINED":
                    if actual_object_type.upper() == part_type_upper:
                        return obj
    return None

def find_bridge_subpart(bridge_part, subpart_type):
    '''
    
    '''
    if not bridge_part.IsDecomposedBy:
        return None
    subpart_type_upper = subpart_type.upper() if subpart_type else "NOTDEFINED"
    
    for rel in bridge_part.IsDecomposedBy:
        for obj in rel.RelatedObjects:
            if obj.is_a("IfcBridgePart"):
                actual_predefined_type = getattr(obj,"PredefinedType", None)
                actual_object_type = getattr(obj, "ObjectType", None) or ""

                if actual_predefined_type == subpart_type_upper:
                    return obj
                if actual_predefined_type == "USERDEFINED":
                    if actual_object_type.upper() == subpart_type_upper:
                        return obj
    return None

def get_or_create_rel_aggregates(ifc_model,relating_object,owner_history):
    '''
    '''
    if relating_object.IsDecomposedBy:
        for rel in relating_object.IsDecomposedBy:
            if rel.is_a("IfcRelAggregates"):
                return rel
    new_rel = ifc_model.create_entity(
        "IfcRelAggregates",
        GlobalId = ifcopenshell.guid.new(),
        OwnerHistory = owner_history,
        Name = "Aggregation",
        Description = None,
        RelatingObject = relating_object,
        RelatedObjects = []
    )
    return new_rel

def create_bridge_part(ifc_model:ifcopenshell.file,ifc_bridge,bridge_part_type,owner_history):
    '''
    
    '''
    bsdd_bridge_part_type_enum = set(get_ifc_bridge_part_type_enum(URL))
    if bsdd_bridge_part_type_enum:
        
        enum_candidate = bridge_part_type.upper() if bridge_part_type else "NOTDEFINED"

        if enum_candidate in bsdd_bridge_part_type_enum:
            predefined_type = enum_candidate
            object_type = None
        else:
            predefined_type ="USERDEFINED"
            object_type = bridge_part_type

        new_bridge_part = ifc_model.create_entity(
            "IfcBridgePart",
            GlobalId = ifcopenshell.guid.new(),
            OwnerHistory =owner_history,
            Name = str(bridge_part_type.title()),
            Description = None,
            ObjectType = object_type,
            PredefinedType = predefined_type
        )

        # Create the IfcRelAggregates that links the bridge to the new part
        rel_agg = get_or_create_rel_aggregates(ifc_model, ifc_bridge,owner_history)
        rel_agg.RelatedObjects = list(rel_agg.RelatedObjects)+[new_bridge_part]
        return new_bridge_part

def create_bridge_subpart(ifc_model:ifcopenshell.file,bridge_part,bridge_subpart_type,owner_history):
    '''

    '''
    bsdd_bridge_part_type_enum = set(get_ifc_bridge_part_type_enum(URL))
    if bsdd_bridge_part_type_enum:
        enum_candidate = bridge_subpart_type.upper() if bridge_subpart_type else "NOTDEFINED"

        if enum_candidate in bsdd_bridge_part_type_enum:
            predefined_type = enum_candidate
            object_type = None
        else:
            predefined_type = "USERDEFINED"
            object_type = bridge_subpart_type

        new_bridge_subpart = ifc_model.create_entity(
            "IfcBridgePart",
            GlobalId = ifcopenshell.guid.new(),
            OwnerHistory = owner_history,
            Name = str(bridge_subpart_type.title()) if bridge_subpart_type else "Unknown",
            Description = None,
            ObjectType = object_type,
            PredefinedType = predefined_type
        )
        #Create the IfcRelAggregates that links the bridge_subpart to the bridge_part

        rel_agg = get_or_create_rel_aggregates(ifc_model,bridge_part, owner_history)
        rel_agg.RelatedObjects = list(rel_agg.RelatedObjects)+[new_bridge_subpart]
        return new_bridge_subpart

def aggregate_element_in_object (ifc_model:ifcopenshell.file,container,element,owner_history):
    '''
    
    '''
    rel_agg = get_or_create_rel_aggregates(ifc_model, container,owner_history)
    rel_agg.RelatedObjects = list(rel_agg.RelatedObjects)+[element]

def place_element_in_bridge_structure( ifc_model,ifc_bridge,element,bridge_part_type,bridge_subpart_type):
    '''
    
    '''
    owner_history = ifc_model.by_type("IfcOwnerHistory")[0]
    #Find a bridge_part for the element,if not found create one 
    bridge_part = find_bridge_part(ifc_bridge,bridge_part_type)
    if not bridge_part:
        bridge_part = create_bridge_part(ifc_model, ifc_bridge, bridge_part_type, owner_history)
    
    # In case bridge_subpart is requested find it, if not found create one
    if bridge_subpart_type != "-":
        bridge_subpart = find_bridge_subpart(bridge_part,bridge_subpart_type)
        if not bridge_subpart:
            bridge_subpart = create_bridge_subpart(ifc_model, bridge_part, bridge_subpart_type,owner_history)

        aggregate_element_in_object(ifc_model, bridge_subpart, element, owner_history)
    else:
        aggregate_element_in_object(ifc_model,bridge_part,element,owner_history)

if __name__=="__main__":

    ifc_paths = [
        r"C:\00_TECNALIA\01_PROJECTS\TCRINI\00_CODE\RemapBridges\public\ifc_bridges\model01_girder.ifc",
        r"C:\00_TECNALIA\01_PROJECTS\TCRINI\00_CODE\RemapBridges\public\ifc_bridges\model02_tablero.ifc",
        r"C:\00_TECNALIA\01_PROJECTS\TCRINI\00_CODE\RemapBridges\public\ifc_bridges\model03_zubiot.ifc",
        r"C:\00_TECNALIA\01_PROJECTS\TCRINI\00_CODE\RemapBridges\public\ifc_bridges\model04_kit.ifc",
        r"C:\00_TECNALIA\01_PROJECTS\TCRINI\00_CODE\RemapBridges\public\ifc_bridges\model05_berriz-IFC4-IFC4X3.ifc",
        r"C:\00_TECNALIA\01_PROJECTS\TCRINI\00_CODE\RemapBridges\public\ifc_bridges\model06_PrecastAndCastConcrete.ifc",
        r"C:\00_TECNALIA\01_PROJECTS\TCRINI\00_CODE\RemapBridges\public\ifc_bridges\Viadotto Acerno.ifc"
    ]
    input_ifc_path = ifc_paths[0]
    output_path = r"C:\00_TECNALIA\01_PROJECTS\TCRINI\00_CODE\RemapBridges\public\ifc_bridge_test_bridge\output_bridge_part.ifc"
    
    ifc_model = ifcopenshell.open(input_ifc_path)
    logging.info(f"Opened IFC file: {input_ifc_path}")
    # Traverse and modify the IFC model
    ifc_bridge = traverse_ifc_bridge_aggregation(ifc_model)

    test_ifc_elements = ifc_model.by_type("IfcWall")
    bridge_part_type = "SUBSTRUCTURE"
    bridge_subpart_type = "ABUTMENT"

    for el in test_ifc_elements:
        place_element_in_bridge_structure(
            ifc_model=ifc_model,
            ifc_bridge = ifc_bridge,
            element = el,
            bridge_part_type= bridge_part_type,
            bridge_subpart_type = bridge_subpart_type,
            
        )
    ifc_model.write(output_path)
