from pprint import pprint
'''
TCRINI mappings:
    Bridges structured like a building following the schema 2X3
    Mappings for IfcClasses based on classes and attributes
'''
FOUNDATION = [
    {
        "class":"IfcFooting",
        "bridge_subpart":None,
        "description":"Foundation element",
        "conditions":{
            "TOPO_SUPERSTRUCTURE":"BELOW"
        }
    },
    {
        "class":"IfcPile",
        "bridge_subpart": None,
        "description":"Deep foundation pile or micropile",
        "conditions":{
            "TOPO_SUPERSTRUCTURE":"BELOW",
            "GEOM_SHAPE":"VER_LINEAR",
            
        }
    },
    {
        "class":"IfcSlab",
        "bridge_subpart":None,
        "description":"Foundation pile classified as slab ",
        "conditions":{
            "TOPO_SUPERSTRUCTURE":"BELOW",
            "GEOM_SHAPE":"VER_LINEAR",
            "TOPO_ELS_ABOVE":["IfcWall","IfcWallStandardCase"],
            "TOPO_ELS_BELOW":[None]
        }
    },
    {
        "class":"IfcSlab",
        "bridge_subpart":None,
        "description":"Foundation slab cleaning concrete",
        "conditions":{
            "TOPO_SUPERSTRUCTURE":"BELOW",
            "GEOM_SHAPE":["HOR_LINEAR","HOR_PLANAR"],
            "TOPO_ELS_ABOVE":["IfcFooting","IfcSlab"]
        }
    },
    {
        "class":"IfcSlab",
        "bridge_subpart":None,
        "description":"Foundation slab classified as IfcSlab",
        "conditions":{
            "TOPO_SUPERSTRUCTURE":"BELOW",
            "TOPO_ELS_ABOVE":["IfcColumn","IfcWall","IfcWallStandardCase"],
            "TOPO_ELS_BELOW":["IfcSlab",None]
        }
    },
    {
        "class":"IfcEarthWorksFill",
        "bridge_subpart":None,
        "description":"Foundation earth movement",
        "conditions":{
            "TOPO_SUPERSTRUCTURE":"BELOW"
        }
    }
]
SUBSTRUCTURE =[
    {
        "class":"IfcColumn",
        "bridge_subpart":"PIER",
        "description":"Substructure pier column",
        "conditions":{
            "TOPO_SUPERSTRUCTURE":["BELOW","SUPERSTRUCTURE"]
        }
    },
    {
        "class":"IfcWall",
        "bridge_subpart":"PIER",
        "description":"Pier Wall",
        "conditions":{
            "TOPO_SUPERSTRUCTURE":"BELOW",
            "TOPO_ELS_ABOVE":["IfcBeam","IfcBuildingElementProxy"],
            "TOPO_ELS_BELOW":["IfcWall","IfcFooting"]
        }
    },
    {
        "class":"IfcWall",
        "bridge_subpart":"ABUTMENT",
        "description":"Abutment predefined type retaining wall",
        "conditions":{
            "TOPO_SUPERSTRUCTURE":["BELOW","SUPERSTRUCTURE"],
            "SEMA_PTYPE":"RETAININGWALL"
            
        }
    },
    {
        "class":"IfcWallStandardCase",
        "bridge_subpart":"ABUTMENT",
        "description":"Abutment IfcWallStandardCase",
        "conditions":{
            "TOPO_SUPERSTRUCTURE":["BELOW","SUPERSTRUCTURE"],
            "TOPO_ELS_BELOW":["IfcWall","IfcWallStandardCase", "IfcSlab","IfcFooting","IfcBeam",None],
            "TOPO_ELS_ABOVE":["IfcWall", "IfcSlab", "IfcBeam","IfcBuildingElementProxy"],
            "TOPO_REL_X":lambda bridge_length: (
                {"min":0.4,"max":1}if bridge_length <50 else
                {"min":0.5,"max":1}if bridge_length <100 else
                {"min":0.7,"max":1 }
                )
        }
    },
    {
        "class":"IfcWall",
        "bridge_subpart":"ABUTMENT",
        "description":"Abutment IfcWall without classification",
        "conditions":{
            "TOPO_SUPERSTRUCTURE":["BELOW","SUPERSTRUCTURE"],
            "TOPO_ELS_BELOW":["IfcWall", "IfcSlab","IfcFooting"],
            "TOPO_ELS_ABOVE":["IfcWall","IfcWallStandardCase", "IfcSlab", "IfcBeam","IfcBuildingElementProxy"],
            "TOPO_REL_X":lambda bridge_length: (
                {"min":0.4,"max":1}if bridge_length <50 else
                {"min":0.5,"max":1}if bridge_length <100 else
                {"min":0.7,"max":1 }
            ),
        }
    },
    {
        "class":"IfcSlab",
        "bridge_subpart":"ABUTMENT",
        "description":"Abutment IfcSlab under or on top of a Wall",
        "conditions":{
            "TOPO_SUBSTRUCTURE":["BELOW"],
            "TOPO_ELS_BELOW":["IfcWall","IfcSlab","IfcFooting","IfcBeam"],
            "TOPO_ELS_ABOVE":["IfcWall","IfcSlab","IfcBeam","IfcOpeningElement"],
            "TOPO_REL_X":lambda bridge_length: (
                {"min":0.4,"max":1}if bridge_length <50 else
                {"min":0.6,"max":1}if bridge_length <100 else
                {"min":0.7,"max":1 })
        }
    },
]
SUPERSTRUCTURE = [
    {
        "class":"IfcSlab",
        "bridge_subpart":"DECK",
        "description":"Deck Slab",
        "conditions":{
            "TOPO_SUPERSTRUCTURE":["SUPERSTRUCTURE"],
        }
    },
    {
        "class":"IfcBeam",
        "bridge_subpart":"DECK",
        "description":"De",
        "conditions":{
            "TOPO_SUPERSTRUCTURE":["SUPERSTRUCTURE"],
        }
    },
    {
        "class":"IfcBuildingElementProxy",
        "bridge_subpart":"DECK",
        "description":"Missclassified IfcBeam or similar",
        "conditions":{
            "TOPO_SUPERSTRUCTURE":["SUPERSTRUCTURE"],
            "GEOM_SHAPE":["HOR_LINEAR"]
        }
    },
    {
        "class":"IfcMember",
        "bridge_subpart":"DECK",
        "description":"Deck as IfcMember",
        "conditions":{
            "TOPO_SUPERSTRUCTURE":"SUPERSTRUCTURE",
            "GEOM_SHAPE":["HOR_LINEAR"]
        }
    }
]

CONNECTION = [
    {
        "class":"IfcBuildingElementProxy",
        "bridge_subpart":"SUPPORT",
        "description":"Missclassified Support element IfcBearing or similar",
        "conditions":{
            "TOPO_SUPERSTRUCTURE":["SUPERSTRUCTURE","BELOW"],
            "GEOM_SHAPE":["NEU_PRISM"]
        }
    },
    {
        "class":"IfcBuildingElementProxy",
        "bridge_subpart":"SUPPORT",
        "description":"Miscclasified support element IfcPlate or similar",
        "conditions":{
            "TOPO_SUPERSTRUCTURE":["SUPERSTRUCTURE","BELOW"],
            "GEOM_SHAPE":["HOR_PLANAR","VER_PLANAR"]
        }
    },
    {
        "class":"IfcBearing",
        "bridge_subpart":"SUPPORT",
        "description":"IfcBearing acting as a Support element",
        "conditions":{
            "GEOM_SHAPE":["HOR_LINEAR","NEU_PRISM"]
        }
    },
    {
        "class":"IfcPlate",
        "bridge_subpart":"SUPPORT"
    },
    {
        "class":"IfcDiscreteAccessory",
        "bridge_subpart":"JOINT",
        "description":"",
        "conditions":{
            "SEMA_PTYPE":["EXPANSION_JOINT_DEVICE"]
        }
    }
]
EQUIPMENT = [ 
    {
        "class":"IfcRailing",
        "bridge_subpart":"SECURITY"
    },
    {
        "class":"IfcBuildingElementProxy",
        "bridge_subpart":"PLATFORM",
        "description":"Missclassified Ifc Element, platform or similar",
        "conditions":{
            "TOPO_SUPERSTRUCTURE":"ABOVE",
            "GEOM_SHAPE":"HOR_PLANAR"
        }
    },
    {
        "class":"IfcSlab",
        "bridge_subpart":"PLATFORM",
        "description":"IfcSlab part of platform",
        "conditions":{
            "TOPO_SUPERSTRUCTURE":["ABOVE"],
            "TOPO_ELS_ABOVE":[None,"IfcRailing","IfcFlowTerminal"]
        }

    },
    {
        "class":"IfcFlowTerminal",
        "bridge_subpart":"SERVICES",
        "conditions":{
            "GEOM_SHAPE":"VER_LINEAR"
        }
    },
    {
        "class":"IfcDiscreteAccessory",
        "bridge_subpart":""
    },
    {
        "class":"IfcStair",
        "bridge_subpart":"CIRCULATION"
    },
    {
        "class":"IfcRamp",
        "bridge_part":"CIRCULATION"
    }

]

BRIDGE_PART_SECTIONS = [("FOUNDATION", FOUNDATION),
                        ("SUBSTRUCTURE", SUBSTRUCTURE),
                        ("SUPERSTRUCTURE", SUPERSTRUCTURE),
                        ("CONNECTION", CONNECTION),
                        ("EQUIPMENT", EQUIPMENT)]

# Unify mappings dynamically
BRIDGE_PARTS = [
    {
        "bridge_part": section_title,  # Dynamically add the title
        **item,  
    }
    for section_title, section_items in BRIDGE_PART_SECTIONS
    for item in section_items
]