import ifcpatch 
import ifcopenshell
import os
import time


def version_migration(input_path,version:str):
    updated_model = ifcpatch.execute({
        "input":input_path,
        "file": ifcopenshell.open(input_path),
        "recipe": "Migrate",
        "arguments": [version],
    })
    return updated_model

def migrate_to_ifc4(input_path):
    try:
        output = ifcpatch.execute({
            "input":input_path,
            "file": ifcopenshell.open(input_path),
            "recipe":"Migrate",
            "arguments":["IFC4"]
        })
        directory = os.path.dirname(input_path)
        base_name = os.path.basename(input_path)
        name = os.path.splitext(base_name)[0]
        new_base_name = f'{name}_ifc4.ifc'

        ifc4_directory = os.path.join(directory,new_base_name)
        updated_ifc4_path = output.write(ifc4_directory)

        return ifc4_directory
    
    except Exception as e:
        print(e)
        
def migrate_to_ifc4X3(input_path):
    try:
        output = ifcpatch.execute({
            "input":input_path,
            "file": ifcopenshell.open(input_path),
            "recipe":"Migrate",
            "arguments":["IFC4X3"]
        })
        directory = os.path.dirname(input_path)
        base_name = os.path.basename(input_path)
        name = os.path.splitext(base_name)[0]
        new_base_name = f'{name}_ifc4X3.ifc'
        
        ifc4x3_directory = os.path.join(directory,new_base_name)
        updated_ifc4X3_path = output.write(ifc4x3_directory)

        return ifc4x3_directory
    except:
        print("Not valid model")

def check_ifc_version(ifc_model:ifcopenshell.file,ifc_version:str):
    try:
        ifc_model_version=  ifc_model.schema_identifier
        if ifc_model_version in ifc_version:
            return True
        else: 
            return False
    except Exception as e:
        print(e)
        return False
    
def migrate_input_to_4X3(input_path):
    input_ifc_model = ifcopenshell.open(input_path)
    
    #Check if IfcShema version is IFC2x3 and migrate to IFC4
    if check_ifc_version(input_ifc_model,"IFC2X3"):
        print("Input IFC 2x3 version detected")
        temporary_ifc_4_path = migrate_to_ifc4(input_path)
        temporary_ifc_4_model = ifcopenshell.open(temporary_ifc_4_path)
        print("IFC Successfully converted from 2x3 to 4")

        # If Converted model version is IFC4 convert it further to IFC4X3
        if check_ifc_version(temporary_ifc_4_model,"IFC4"):
            converted_ifc_4X3_path = migrate_to_ifc4X3(temporary_ifc_4_path)
            print("IFC Successfully converted from 4 to 4X3")
            return converted_ifc_4X3_path
        
    #If input model is already IFC4
    elif check_ifc_version(input_ifc_model,"IFC4"):
        print("Input IFC 4 version detected")
        converted_ifc_4X3_path = migrate_to_ifc4X3(input_path)
        print("IFC Succesfully coverted from 4 to 4x3")
        return converted_ifc_4X3_path
    
    #Rest of the cases such as IFC4X2
    elif check_ifc_version(input_ifc_model,"IFC4X2"):
        print("Unrecognised Ifc4X2")
    elif check_ifc_version(input_ifc_model,"4X3"):
        print("Input file is already in 4x3 version")
        input_path_4X3 = input_path
        return input_path_4X3    
        

if __name__=="__main__":
    input_path = r"C:\00_TECNALIA\01_PROJECTS\TCRINI\00_CODE\RemapBridges\public\ifc_bridges2x3\model02_tablero_2x3.ifc"
    input_path2 = r"C:\00_TECNALIA\01_PROJECTS\TCRINI\00_CODE\RemapBridges\public\ifc_bridges2x3\model02_4.ifc"

    output_directory = r"C:\00_TECNALIA\01_PROJECTS\TCRINI\00_CODE\RemapBridges\public\ifc_bridges4"
    
    output_path = r"C:\00_TECNALIA\01_PROJECTS\TCRINI\00_CODE\RemapBridges\public\ifc_bridges4\model02_4.ifc"
    output_path2 = r"C:\00_TECNALIA\01_PROJECTS\TCRINI\00_CODE\RemapBridges\public\ifc_bridges4\model02_4X3.ifc"

    start_time = time.time()

    converted_input_path = migrate_input_to_4X3(input_path)
    ifc_model = ifcopenshell.open(converted_input_path)
    print(ifc_model.schema_identifier)

    end_time = time.time()
    elapsed_time = end_time-start_time
    print(f'{elapsed_time} seconds')

